/***************************************************************************
 *   Copyright (C) 2005 by Sebastian Beck                                  *
 *   sebastian.bw@freenet.de                                                    
 *   
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
  /*01, april, 2007
 
KHistoryDoubleNumInput extends KDoubleNumInput class by a KCompletionBox for history input selection.

If the Mousecursor is over KHistoryDoubleNumInput:
 -The mouse will be grabbed if KCompletionBox has min. one entry.
 -MouseTracking is then active. If mouse position is out of this Widget the
  mouse will be released for QT.
 -The pop-up menu will be shown if you press a mousebutton and KCompletionBox
  has min. one entry.
 -If the KCompletionBox has min. one entry it uses one click for pop-up and
  one for close, until KHistoryDoubleNumInput works like KDoubleNumInput
  again. The second click can be double to select all for overwriting.
 -An entry will be stored if you enter something with the keyboard that's not
  exactly greater or less than one step.
 -Values entered with the SpinBox won't be stored because of that.
 -If KCompletionBox has no entry, KHistoryDoubleNumInput will behave like
  KDoubleNumInput.
 -By pressing a mouse key and moveing on a KHistoryDoubleNumInput the
  KCompletionBox won't be activated at all. So drag and drop is possible.
 
The SpinBox value can't be compared with the value I stored plus/minus the 
step of the SpinBox, for filtering, so I have to make them proper.
 
I think this is the most compact form for history extension, and it is good to 
handle. I was thinking about an overloaded version with a pulldown button for 
history but that would have made KHistoryDoubleNumInput too wide.

It's in separate class files, that you can test and change it better and I 
think knuminput gets now a little too big.
---------------------------------------------------

This class relays on KDoubleNumInput's widget (area) and on QT (mouse)events. 
So this class will work on all versions of KDoubleNumInput and KDE.
So this class can be used with KDE-4.x, KDE-3.x, KDE-2.x and should work on each app. that has a KDoubleNumInput and wants a history.

 Dependences:
- KDoubleNumInput::value()
- KDoubleNumInput::setValue()
- KDoubleNumInput::valueChanged( double ) [signal]
- KCompletionBox::activated( const QString & ) [signal]
- KCompletionBox::m_step [property]
- QWidget::releaseMouse()
- QWidget::enterEvent( QEvent * )
- QWidget::leaveEvent( QEvent * )
- QWidget::setFocus()
- QWidget::clearFocus()
- QWidget::mousePressEvent( QMouseEvent * )
- QWidget::mouseMoveEvent( QMouseEvent * )
- QWidget::mouseReleaseEvent( QMouseEvent * )
- QString::toDouble()

 */
 
 
#include "khistorydoublenuminput.h"
#include <kcompletionbox.h>
#include <knuminput.h>
#include <qevent.h>
#include <math.h>

KHistoryDoubleNumInput::KHistoryDoubleNumInput(QWidget *parent, const char *name)
 : KDoubleNumInput(parent, name)
	{
	cb = new KCompletionBox( this );
	cb->hide();
	cb->setMinimumWidth( 20 );
	last = value();
	setMouseTracking( TRUE );
	
	connect( cb, SIGNAL( activated( const QString & ) ), this, SLOT( insert( const QString & ) ) ) ;
 	connect( this, SIGNAL( valueChanged( double ) ), this, SLOT( newItem( double ) ) );
	}

KHistoryDoubleNumInput::~KHistoryDoubleNumInput()
	{
	}

void KHistoryDoubleNumInput::enterEvent( QEvent * )
	{
	if( cb->count() > 0)
		{
		grabMouse();
		setFocus();
		}
	}

void KHistoryDoubleNumInput::leaveEvent( QEvent * )
	{
 	releaseMouse();
	}

void KHistoryDoubleNumInput::mousePressEvent( QMouseEvent * )
	{
	releaseMouse();
	cb->popup();
	}

void KHistoryDoubleNumInput::mouseMoveEvent( QMouseEvent *e )
	{
	if( 0 > e->x() || e->x() > width() )
		{
		releaseMouse();
		}
	
	if( 0 > e->y() || e->y() > height() )
		{
		clearFocus();
		releaseMouse();
		}
	}

void KHistoryDoubleNumInput::mouseReleaseEvent( QMouseEvent * )
	{
	releaseMouse();
	}

void KHistoryDoubleNumInput::insert( const QString &s )
	{
	setValue( s.toDouble() );
	releaseMouse();
	}

void KHistoryDoubleNumInput::newItem( double )
	{// round - because the double values should have the same length for comparison,  that only key inputs are listed
	if( round(value()*exp10(precision())) != round((last + m_step)*exp10(precision())) && round(value()*exp10(precision())) != round((last - m_step)*exp10(precision())) && ! cb->items().contains( QString::number( value(), 'f', precision() ) ) )
		cb->insertItem( QString::number( value(), 'f', precision() ) );
	
	last = value();
	}


// #include "khistorydoublenumInput.moc"

