/***************************************************************************
 *   Copyright (C) 2005 by Sebastian Beck                                  *
 *   sebastian.bw@freenet.de                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KONTURDIALOG_H
#define KONTURDIALOG_H

#include "kontur.h"
#include "k_linie.h"
#include "k_kreis.h"
#include "k_liniex.h"
#include "k_liniey.h"
#include "vorschub.h"
#include "k_ellipse.h"
#include "kf_linear.h"
#include "kf_kreis.h"
#include "kf_ellipse.h"
#include "k_plus-.h"

#include <qpainter.h>
#include <qwmatrix.h>
#include <qstring.h>

#include"khistorydoublenuminput.h"


class Konturdialog: public kontur 
	{ Q_OBJECT
	
	public:
		Konturdialog(QWidget *parent = 0, const char *name = 0);
		
		virtual ~Konturdialog();
		
		K_Linie *KL;
		K_Kreis *KK;
		K_Liniex *KLx;
		K_Liniey *KLy;
		Vorschub *KVrs;
		K_Ellipse *KEl;
		KF_Linear *KFL;
		KF_Kreis *KFK;
		KF_Ellipse *KFE;
		K_Aendern *KA2;
		K_Aendern *KA3;
		
		
		struct KonturData
			{
			char element; 	// 1=LINIE 2= LINIEX 3=LINIEY 4=LINIEZ 5=KREIS 6=ELLIPSE
			int line_starts;
			int line_ends;
			double x1;
			double y1;
			double x2;
			double y2;
			double dx;
			double dy;
			double dl;
			double w;
			double we;
			double r;
			double r2;
			double mx;
			double my;
			int VALID;		// bit0=1 element, bit1=1 dx ...
			};
		
		#define LINIE		1
		#define LINIEX		2
		#define LINIEY 	3
		#define LINIEZ		4
		#define KREIS		5
		#define ELLIPSE	6
		
		#define ELEMENT_VALID	1
		#define DX_PRESENT		2
		#define DY_PRESENT		4
		#define DL_PRESENT		8
		#define X1_PRESENT		16
		#define Y1_PRESENT			32
		#define X2_PRESENT		64
		#define Y2_PRESENT			128
		#define W_PRESENT			256	// Angel of this element
		#define WE_PRESENT		512	// Angel between this and the elemnt before
		#define R_PRESENT			1024
		#define R2_PRESENT			2048
		#define MX_PRESENT		4096
		#define MY_PRESENT		8192
		
		
		
		QPainter painter;
		QWMatrix wm;
		bool SOLVED;
		int x, y, kn, kfn, i, exitcode, elements_count;
		double W, X, Y;
		QString kontur_file, expr;
		KonturData kd1, kd2, kd3, *kd_activ, *kd, *kd_minus, *kd_plus;

	private slots:
		void resizeEvent( QResizeEvent* );
		void paintEvent( QPaintEvent* );
		void closeEvent( QCloseEvent* );
		void save();
		void calc();
		void make();
		bool cldx();
		bool cldy();
		bool clw();
		bool Ldx_x();
		bool Ldx_y();
		bool Ldy_y();
		bool Ldy_x();
		bool Ldx_w();
		bool Ldx_we();
		bool Ldy_w();
		bool Ldy_we();
	
	public slots:
		virtual void linie_clicked();
		virtual void kreis_clicked();
		virtual void vorschub_clicked();
		virtual void ellipse_clicked();
		virtual void f_linear_clicked();
		virtual void f_kreis_clicked();
		virtual void f_ellips_clicked();
		virtual void test_clicked();
		virtual void insert_clicked();
		virtual void file_choice_clicked();
		virtual void save_clicked();
		virtual void tab_changed();
		virtual void liniey_clicked();
		virtual void liniex_clicked();
		virtual void aendern_3d_clicked();
		virtual void aendern_2d_clicked();
		virtual void draw_editor();
		virtual void file_entered();
		
		virtual void k_kreis_verwenden_clicked();//   parentWidget()
		virtual void k_ellipse_verwenden_clicked();//   parentWidget()
		virtual void kf_ellipse_anwenden_clicked();//   parentWidget()
		virtual void kf_kreis_anwenden_clicked();//   parentWidget()
		virtual void kf_linear_anwenden_cklicked();//   parentWidget()
		virtual void k_linie_verwenden_clicked();//   parentWidget()
		virtual void k_liniex_verwenden_clicked();//   parentWidget()
		virtual void k_liniey_verwenden_clicked();//   parentWidget()
		virtual void k_aendern_ausfuehren_clicked();//   parentWidget()
		virtual void vs_schlichten_einfuegen_clicked();//   parentWidget()
		virtual void vs_normal_einfuegen_clicked();//   parentWidget()
		virtual void vs_max_z_einfuegen_clicked();//   parentWidget()
		virtual void vs_max_y_einfuegen_clicked();//   parentWidget()
		virtual void vs_max_x_einfuegen_clicked();//   parentWidget()
		virtual void vs_eintauchen_einfuegen_clicked();//   parentWidget()
		virtual void vs_bohren_einfuegen_clicked();//   parentWidget()
		virtual void k_linie_py2_changed();//   parentWidget()
		virtual void k_linie_px2_changed();//   parentWidget()
		virtual void k_linie_py1_changed();//   parentWidget()
		virtual void k_linie_px1_changed();//   parentWidget()
		virtual void k_linie_dx_changed();//   parentWidget()
		virtual void k_linie_dy_changed();//   parentWidget()
		virtual void k_linie_dl_changed();//   parentWidget()
		virtual void k_linie_w_changed();//   parentWidget()
		virtual void k_linie_we_changed();//   parentWidget()
		virtual void k_liniey_py2_changed();//   parentWidget()
		virtual void k_liniey_py1_changed();//   parentWidget()
		virtual void k_liniex_px2_changed();//   parentWidget()
		virtual void k_liniex_px1_changed();//   parentWidget()
		virtual void k_ellipse_pmy_changed();//   parentWidget()
		virtual void k_ellipse_pmx_changed();//   parentWidget()
		virtual void k_ellipse_ws2_changed();//   parentWidget()
		virtual void k_ellipse_ws1_changed();//   parentWidget()
		virtual void k_kreis_pmy_changed();//   parentWidget()
		virtual void k_kreis_pmx_changed();//   parentWidget()
		virtual void k_kreis_ws2_changed();//   parentWidget()
		virtual void k_kreis_winkel1_changed();//   parentWidget()
		virtual void k_kreis_winkel2_changed();//   parentWidget()
		virtual void k_kreis_ws1_changed();//   parentWidget()


	};

#endif
